/*
 * robot.cpp
 *
 *  Created on: May 16, 2012
 *      Author: Daniel
 */
#include "drivers/pwm.h"
#include "robot.hpp"

#include "../System/lpc214x.h"
#include "../System/sysConfig.h"	// Need PCLK
#include "../FreeRTOS/FreeRTOS.h"
#include "../FreeRTOS/Semphr.h"
#include "../FreeRTOS/task.h"

void initializeRobot(int pwmPulseWidth)
{
	initialize_PWM(pwmPulseWidth);//set the pwm

	PINSEL0 &= ~(3<<12);  // select direction (P0.6) and clear bits
	PINSEL1 &= ~(3<<12);  //select direction (P0.22) and clear bits
	PINSEL1 &= ~(3<<26);  //select direction (P0.29) and clear bits
	PINSEL1 &= ~(3<<28); //select P0.30, to be used to imitate pwm

	IODIR0 |= (1<<6); // set dir 00
	IODIR0 |= (1<<22);
	IODIR0 |= (1<<29);
	IODIR0 |= (1<<30);

	IOPIN0 |= (1<<30); //for pwm 0% dutycycle
}

void walkForward(int pwmPulseWidth)
{
	//setup
	setDutycycle_PWM2(0, pwmPulseWidth);
	setDutycycle_PWM5(0, pwmPulseWidth);
	IOPIN0 &= ~(1<<30); //for pwm 100% dutycycle

	//walk
	IOPIN0 |= (1<<29);  //tilt right
	vTaskDelay(200);

	IOPIN0 |= (1<<6);   // left forward
	IOPIN0 &= ~(1<<22); //right backward

	vTaskDelay(400);

	IOPIN0 &= ~(1<<29); //tilt left
	vTaskDelay(200);

	IOPIN0 &= ~(1<<6);   // left backward
	IOPIN0 |= (1<<22);  //right forward

	vTaskDelay(400);
}

void walkBackward(int pwmPulseWidth)
{
	//setup
	setDutycycle_PWM2(0, pwmPulseWidth);
	setDutycycle_PWM5(0, pwmPulseWidth);
	IOPIN0 &= ~(1<<30); //for pwm 100% dutycycle

	//walk
	IOPIN0 &= ~(1<<29); //tilt left
	vTaskDelay(200);

	IOPIN0 |= (1<<6);   // left forward
	IOPIN0 &= ~(1<<22); //right backward

	vTaskDelay(400);

	IOPIN0 |= (1<<29);  //tilt right
	vTaskDelay(200);

	IOPIN0 &= ~(1<<6);   // left backward
	IOPIN0 |= (1<<22);  //right forward

	vTaskDelay(400);
}

void turnLeft(int pwmPulseWidth)
{
	//setup
	setDutycycle_PWM2(0, pwmPulseWidth);
	setDutycycle_PWM5(0, pwmPulseWidth);
	IOPIN0 &= ~(1<<30); //for pwm 100% dutycycle

	//walk
	IOPIN0 &= ~(1<<6);   // left backward
	IOPIN0 |= (1<<22);  //right forward

	vTaskDelay(400);

	IOPIN0 &= ~(1<<29); //tilt left

	vTaskDelay(200);

	IOPIN0 |= (1<<29);  //tilt right

	vTaskDelay(200);

	IOPIN0 |= (1<<6);   // left forward
	IOPIN0 &= ~(1<<22); //right backward

	vTaskDelay(400);
}

void turnRight(int pwmPulseWidth)
{
	//setup
	setDutycycle_PWM2(0, pwmPulseWidth);
	setDutycycle_PWM5(0, pwmPulseWidth);
	IOPIN0 &= ~(1<<30); //for pwm 100% dutycycle

	//walk
	IOPIN0 |= (1<<6);   // left forward
	IOPIN0 &= ~(1<<22); //right backward

	vTaskDelay(400);

	IOPIN0 |= (1<<29);  //tilt right

	vTaskDelay(200);

	IOPIN0 &= ~(1<<29); //tilt left

	vTaskDelay(200);

	IOPIN0 &= ~(1<<6);   // left backward
	IOPIN0 |= (1<<22);  //right forward

	vTaskDelay(400);
}

void stopRobot(int pwmPulseWidth)
{
	//stop
	setDutycycle_PWM2(100, pwmPulseWidth);
	setDutycycle_PWM5(100, pwmPulseWidth);
	IOPIN0 |= (1<<30); //for pwm 0% dutycycle
}
