/*
 * pwm.c
 *PWM Functions
 */
#include "pwm.h"

#include "../System/lpc214x.h"
#include "../System/sysConfig.h"	// Need PCLK
#include "../FreeRTOS/FreeRTOS.h"
#include "../FreeRTOS/Semphr.h"
#include "../FreeRTOS/task.h"

void initialize_PWM(int pulseWidth)
{//function to set the pwm parameters
	rprintf("initializing\n");
	PINSEL0 &= ~(3<<14);  // clear bits of P0.7 for pwm2
	PINSEL1 &= ~(3<<10); //clear bits of P0.21 for pwm5
	PINSEL0 |= (2<<14); // set to PWM2
	PINSEL1 |= (1<<10); //set to PWM5

	rprintf("Set PWM settings\n");

	PWM_TCR=PWM_TCR_CR; //tcr is reset
	PWM_PR=0; //pwm tc increments every pclk
	PWM_MR0 = (configCPU_CLOCK_HZ/pulseWidth); //configCPU_CLOCK_Hz = 48MHz
	PWM_MCR |= PWM_MCR_MR0R;//reset MCR
	PWM_PCR |= PWM_PCR_ENA2;//select pwm enable 2
	//PWM_PCR |= PWM_PCR_SEL2; //select comparison with MR1
	PWM_PCR |= PWM_PCR_ENA5;//select pwm enable 5
	PWM_TCR = (PWM_TCR_CE | PWM_TCR_PWME); // tcr is enable counter and enable pwm

	rprintf("PWM should work\n");

	//PWM_MR1 = (configCPU_CLOCK_HZ/pulseWidth); //set MR1, PWM is based off it
	PWM_MR2 = ((configCPU_CLOCK_HZ/pulseWidth) * (100))/100;//duty cycle check

	PWM_LER = PWM_LER_M2L; //enable M2 latch

	PWM_MR5 = ((configCPU_CLOCK_HZ/pulseWidth) * (100))/100;//duty cycle check

	//PWM_LER = PWM_LER_M1L; //enable M1 latch

	PWM_LER = PWM_LER_M5L; //enable M5 latch
}

void setDutycycle_PWM2(int dutycycle, int pulseWidth)
{//function to set the dutycycle
	if(dutycycle<0)
		dutycycle = 0;
	if(dutycycle>100)
		dutycycle = 100;

	PWM_MR2 = ((configCPU_CLOCK_HZ/pulseWidth) * (dutycycle))/100;
	PWM_LER = PWM_LER_M2L; //enable M2 latch
}

void setDutycycle_PWM5(int dutycycle, int pulseWidth)
{//function to set the dutycycle
	if(dutycycle<0)
		dutycycle = 0;
	if(dutycycle>100)
		dutycycle = 100;

	PWM_MR5 = ((configCPU_CLOCK_HZ/pulseWidth) * (dutycycle))/100;
	PWM_LER = PWM_LER_M5L; //enable M5 latch

}
