#include "osHandles.h"              // Includes all OS Files

#include <stdlib.h>                 // atoi()
#include <stdio.h>                  // printf() or iprintf()
#include <string.h>                 // strtok()

#include "System/cpu.h"             // Initialize CPU Hardware

#include "drivers/watchdog.h"
#include "drivers/uart0.h"       	// Initialize UART
#include "../drivers/sta013.h"    // STA013 Mp3 Decoder
#include "../drivers/pcm1774.h"   // PCM 1774 DAC
#include "../drivers/i2c.h"

#include "utils/rprintf.h"			// Reduced printf.  STDIO uses lot of FLASH space & Stack space
#include "utils/timer.hpp"
#include "../utils/vector.hpp"

#include "General/robot.hpp"		//includes functions to control the robot

#include "fat/diskio.h"

#include "../General/userInterface.hpp"	// User interface functions to interact through UART
#include "../General/robotTask.hpp"

#include "../fat/ff.h"
#include "drivers/uart1.h"

void FatFsDiskTimer(void* p)
{
	disk_timerproc();
}

/* INTERRUPT VECTORS:
 * 0:    OS Timer Tick
 * 1:    Not Used
 * 2:    UART0 Interrupt
 * 3:    Not Used
 * 4:    I2C0 Interrupt
 * 5-16: Not Used
 */
int main (void)
{
	OSHANDLES SysHandles;        // Should contain all OS Handles

	cpuSetupHardware();          // Setup PLL, enable MAM etc.
	watchdogDelayUs(1000*1000);  // Some startup delay
	uart0Init(38400, 128);       // 128 is size of Rx/Tx FIFO
	uart1Init(9600, 128); 		// uart1 initialization for wifi

	// Use polling version of uart0 to do printf/rprintf before starting FreeRTOS
	rprintf_devopen(uart0PutCharPolling);
	cpuPrintMemoryInfo();

	// Open interrupt-driven version of UART0 Rx/Tx
	rprintf_devopen(uart0PutChar);
	watchdogDelayUs(1000*1000);

	int pwmPulseWidth = 20000;
	initializeRobot(pwmPulseWidth);

	/** Create timer needed for SD Card I/O */
	//Timer diskTimer(FatFsDiskTimer, 10, TimerPeriodic);
	//diskTimer.start();

	SysHandles.queue.userCommand = xQueueCreate(1, 16);

	// Use the WATERMARK command to determine optimal Stack size of each task (set to high, then slowly decrease)
	// Priorities should be set according to response required
	if (
		pdPASS != xTaskCreate( uartUI, (signed char*)"Uart UI", STACK_BYTES(1024*6), &SysHandles, PRIORITY_LOW,  &SysHandles.task.userInterface )
		||
		pdPASS != xTaskCreate( robotTask, (signed char*)"Robot Task", STACK_BYTES(1024*6), &SysHandles, PRIORITY_LOW,  &SysHandles.task.robotTask )
	){
		rprintf("ERROR:  OUT OF MEMORY: Check OS Stack Size or task stack size.\n");
	}

	// Start FreeRTOS to begin servicing tasks created above, vTaskStartScheduler() will not "return"
	rprintf("\n-- Starting FreeRTOS --\n");
	vTaskStartScheduler();

	// In case OS is exited:
	rprintf_devopen(uart0PutCharPolling);
	rprintf("ERROR: Unexpected OS Exit!\n");

	return 0;
}
